'From Squeak3.4alpha of ''11 November 2002'' [latest update: #5109] on 18 November 2002 at 1:31:44 pm'!
Object subclass: #AccufontLoader
	instanceVariableNames: 'availableFonts '
	classVariableNames: 'Singleton '
	poolDictionaries: ''
	category: 'Graphics-Text'!

!AccufontLoader commentStamp: '<historical>' prior: 0!
---------------------------------------------------------------------------------------------------------
	This code is the first attempt to replace existing Squeak fonts with new ones.
	The first pass replaces all instance of the existing fonts with 'Accu'  versions.
	The second pass loads in additional fonts.  The transcript will show the progress
	on loading the various fonts.  An attempt is made to make existing windows still
	display, even though the font will change - in doing so, there may be duplicates of the
	Accuny font until those windows are closed and the garbage collector scavenges
	them.

	Per the arrangement with Arts Management Systems, Ltd, an attempt has been made
	to encode copyright information into the fonts.  This is done by converting the
	copyright notice as a bitmap in glyph 16rF0, which is unused in these fonts.  Depending
	upon the pointsize of the font, various versions of this notice appear.

	The .sf2 files must be in 'FileDirectory default' because of limitations in the StrikeFont reader.

	Bugs:
		None currently known.

	Submitted by:
		Duane Maxwell, exobox
---------------------------------------------------------------------------------------------------------!


!AccufontLoader methodsFor: 'loading' stamp: 'RAA 1/12/2001 13:12'!
accuFontNamesAndSizes

	| d |
	d _ Dictionary new.
	d add: 'Accuny' -> #(9 10 12 14 18 24).
	d add: 'Accumon' -> #(9 10 12).
	d add: 'Accula' -> #(12 24).
	d add: 'Accushi' -> #(12).
	d add: 'Accusf' -> #(18).
	d add: 'Accumo' -> #(18).
	d add: 'Accuca' -> #(18).
	d add: 'Accuat' -> #(18).
	d add: 'Accuve' -> #(12 14).
	d add: 'Accujen' -> #(9 10 12 14 18 24).
	^d
! !

!AccufontLoader methodsFor: 'loading' stamp: 'RAA 1/12/2001 13:13'!
accuFontReplacementMap

	| d |
	d _ Dictionary new.
	d add: #NewYork10 -> 'Accuny10'.
	d add: #NewYork12 -> 'Accuny12'.
	d add: #NewYork15 -> 'Accuny14'.
	d add: #NewYork18 -> 'Accuny18'.
	d add: #NewYork24-> 'Accuny24'.
	d add: #ComicPlain09 -> 'Accujen09'.
	d add: #ComicPlain10 -> 'Accujen10'.
	d add: #ComicPlain12 -> 'Accujen12'.
	d add: #ComicPlain14 -> 'Accujen14'.
	d add: #ComicBold09 -> 'Accujen09'.
	d add: #ComicBold10 -> 'Accujen10'.
	d add: #ComicBold12 -> 'Accujen12'.
	d add: #ComicBold14 -> 'Accujen14'.
	d add: #ComicBold18 -> 'Accujen18'.
	d add: #Palatino09 -> 'Accuny09'.
	d add: #Palatino10 -> 'Accuny10'.
	d add: #Palatino12 -> 'Accuny12'.
	d add: #Palatino14 -> 'Accuny14'.
	d add: #Palatino24 -> 'Accuny24'.
	^d
! !

!AccufontLoader methodsFor: 'loading' stamp: 'RAA 1/12/2001 13:55'!
availableFonts
"
AccufontLoader singleton availableFonts
"
	| familyName sizes fontName sourceDir file |

	availableFonts ifNotNil: [^availableFonts].
	availableFonts _ Dictionary new.
	sourceDir _ FileDirectory default directoryNamed: 'FontsBeta'.
	self accuFontNamesAndSizes associationsDo: [ :a |
		familyName _ a key.
		sizes _ a value.
		sizes withIndexDo: [ :size :index |
			fontName _ familyName,
				(size < 10 ifTrue: [ '0',size printString] ifFalse: [ size printString]).

			file _ sourceDir readOnlyFileNamed: fontName,'.sf2'.
			file binary.
			availableFonts at: fontName put: file contentsOfEntireFile.
		].
	].
	^availableFonts

! !

!AccufontLoader methodsFor: 'loading' stamp: 'nk 11/18/2002 13:18'!
clearAvailableFonts
	availableFonts _ Dictionary new.! !

!AccufontLoader methodsFor: 'loading' stamp: 'RAA 1/12/2001 14:08'!
installNewAccuFonts

	| familyName sizes fontArray fontName newFont answer blather | 

	answer _ Text new.
	blather _ ' is one of the new fonts. How do you like it?
'.
	self accuFontNamesAndSizes associationsDo: [ :a |
		familyName _ a key.
		sizes _ a value.
		fontArray _ Array new: sizes size.
		sizes withIndexDo: [ :size :index |
			fontName _ familyName,
				(size < 10 ifTrue: [ '0',size printString] ifFalse: [ size printString]).
			newFont _ self makeAccuFontNamed: fontName.
			newFont reset.
			fontArray at: index put: newFont.
			answer _ answer , (
				(fontName,blather) asText addAttribute: (TextFontReference toFont: newFont); yourself
			).
		].
		Transcript show: 'Adding font ',familyName; cr.
		TextConstants at: familyName asSymbol put: (TextStyle fontArray: fontArray).
	].

	"Reset the default font to one of our new ones"
	TextConstants at: #DefaultTextStyle put: (TextConstants at: #Accuny).

	StringHolder new
		textContents: answer;
		openLabel: 'Added to your image'
! !

!AccufontLoader methodsFor: 'loading' stamp: 'RAA 1/12/2001 13:22'!
loadAccuFonts

""

	self replaceExistingFonts.

	self installNewAccuFonts.

! !

!AccufontLoader methodsFor: 'loading' stamp: 'RAA 1/12/2001 13:58'!
makeAccuFontNamed: fontName

	| newFont copyBlock copyright f |

	"build copyright notice bitmaps"
	copyright _ Array new: 3.
	{	(169 asCharacter asString,'ArtsManagementSystemsLtd').
		(169 asCharacter asString,'ArtMgtSysLtd').
		(169 asCharacter asString,'AMSLtd')
		} withIndexDo: [:s :index |
			f _ Form extent: 8@s size.
			s asByteArray withIndexDo: [ :c :row |
				f bits byteAt: row*4-3 put: c ].
			copyright at: index put: f].

	"make a block that returns the correct copyright bitmap for a font"
	copyBlock _ [ :height |
		f _ copyright at: 1.
		height <= 28 ifTrue: [ f _ copyright at: 2].
		height <= 13 ifTrue: [ f _ copyright at: 3].
		f copy: (0@0 extent: 8@height)].

	(newFont _ StrikeFont new) 
		name: fontName;
		readFromStrike2Stream: (
			(RWBinaryOrTextStream with: (self availableFonts at: fontName)) reset; binary
		).

	newFont characterFormAt: 16rF0 asCharacter put: (copyBlock value: newFont height).
	^newFont
! !

!AccufontLoader methodsFor: 'loading' stamp: 'RAA 1/12/2001 13:58'!
replaceExistingFonts

	| i d fontName sz newFont |
	"collect all font references in the current image"
	Smalltalk garbageCollect.
	i _ StrikeFont allInstances.

	"create mapping from existing fonts to their replacements"
	d _ self accuFontReplacementMap.

	"replace fonts currently in use in the image"
	i do: [ :font |
		fontName _ font name asSymbol.
		fontName last isDigit ifFalse: [
			sz _ font pointSize.
			fontName _ (font familyName asString,
				(sz < 10 ifTrue: [ '0',sz printString] ifFalse: [ sz printString]))
				asSymbol].
		(d includesKey: fontName)
			ifTrue: [
				newFont _ self makeAccuFontNamed: (d at: fontName).
				newFont reset.
				Transcript show: 'converting ', fontName, ' to ', newFont name; cr.
				font become: newFont.
				]
			ifFalse: [
				Transcript show: 'cannot find ',fontName; cr.
				]
		].

	"kill the newly orphaned fonts"
	Smalltalk garbageCollect.

	"Remove entries for old fonts in the TextConstants dictionary"
	#(NewYork ComicPlain ComicBold Palatino) do: [ :oldName |
		TextConstants removeKey: oldName asSymbol ifAbsent: []].

! !


!AccufontLoader class methodsFor: 'as yet unclassified' stamp: 'nk 11/18/2002 13:31'!
emptySingleton
	Singleton _ self new.
	Singleton clearAvailableFonts.
	^Singleton
! !

!AccufontLoader class methodsFor: 'as yet unclassified' stamp: 'RAA 1/12/2001 13:24'!
singleton

	^Singleton ifNil: [
		Singleton _ self new.
		Singleton availableFonts.
		Singleton
	].! !


!StrikeFont methodsFor: 'file in/out' stamp: 'RAA 1/11/2001 13:26'!
readFromStrike2: fileName directory: aDirectory

	| file |

	('*.sf2' match: fileName) ifFalse: [self halt.  "likely incompatible"].
	name _ fileName copyUpTo: $. .  "Drop filename extension"
	file _ aDirectory readOnlyFileNamed: fileName.
	file binary.
	[self readFromStrike2Stream: file] ensure: [file close]! !

